@@ -3,8 +3,7 @@
    Tables of information... */
 
 /*
- * Copyright (c) 2011-2012 by Internet Systems Consortium, Inc. ("ISC")
- * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2016 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -199,9 +198,22 @@ static struct option dhcp_options[] = {
 	{ "pxe-client-id", "BX",		&dhcp_universe,  97, 1 },
 #endif
 	{ "uap-servers", "t",			&dhcp_universe,  98, 1 },
+#if defined(RFC4776_OPTIONS)
+        { "geoconf-civic", "X",                 &dhcp_universe, 99, 1 },
+#endif
+#if defined(RFC4833_OPTIONS)
+	{ "pcode", "t",				&dhcp_universe, 100, 1 },
+	{ "tcode", "t",				&dhcp_universe, 101, 1 },
+#endif
 	{ "netinfo-server-address", "Ia",	&dhcp_universe, 112, 1 },
 	{ "netinfo-server-tag", "t",		&dhcp_universe, 113, 1 },
 	{ "default-url", "t",			&dhcp_universe, 114, 1 },
+#if defined(RFC2563_OPTIONS)
+	{ "auto-config", "B",			&dhcp_universe, 116, 1 },
+#endif
+#if defined(RFC2937_OPTIONS)
+	{ "name-service-search", "Sa",		&dhcp_universe, 117, 1 },
+#endif
 	{ "subnet-selection", "I",		&dhcp_universe, 118, 1 },
 	{ "domain-search", "Dc",		&dhcp_universe, 119, 1 },
 	{ "vivco", "Evendor-class.",		&dhcp_universe, 124, 1 },
@@ -220,10 +232,34 @@ static struct option dhcp_options[] = {
 	{ "pxe-undefined-7", "X",		&dhcp_universe, 134, 1 },
 	{ "pxe-undefined-8", "X",		&dhcp_universe, 135, 1 },
 #endif
+#if defined(RFC5192_OPTIONS)
+	{"pana-agent", "Ia",			&dhcp_universe, 136, 1 },
+#endif
+#if defined(RFC5223_OPTIONS)
+	{"v4-lost", "d",			&dhcp_universe, 137, 1 },
+#endif
+#if defined(RFC5417_OPTIONS)
+	{"capwap-ac-v4", "Ia",			&dhcp_universe, 138, 1 },
+#endif
+#if defined(RFC6011_OPTIONS)
+	{ "sip-ua-cs-domains", "Dc",		&dhcp_universe, 141, 1 },
+#endif
+#if defined(RFC6153_OPTIONS)
+	{ "ipv4-address-andsf", "IA",		&dhcp_universe, 142, 1 },
+#endif
+#if defined(RFC6731_OPTIONS)
+        { "rdnss-selection", "BIID",		&dhcp_universe, 146, 1 },
+#endif
 #if 0
 	/* Not defined by RFC yet */
 	{ "tftp-server-address", "Ia",		&dhcp_universe, 150, 1 },
 #endif
+#if defined(RFC7618_OPTIONS)
+	{ "v4-portparams", "BBS",		&dhcp_universe, 159, 1 },
+#endif
+#if defined(RFC7710_OPTIONS)
+	{ "v4-captive-portal", "t",		&dhcp_universe, 160, 1 },
+#endif
 #if 0
 	/* PXELINUX options: defined by RFC 5071 */
 	{ "pxelinux-magic", "BBBB",		&dhcp_universe, 208, 1 },
@@ -231,9 +267,11 @@ static struct option dhcp_options[] = {
 	{ "loader-pathprefix", "t",		&dhcp_universe, 210, 1 },
 	{ "loader-reboottime", "L",		&dhcp_universe, 211, 1 },
 #endif
-#if 0
-	/* Not defined by RFC yet */
-	{ "vss-info", "BX",			&dhcp_universe, 221, 1 },
+#if defined(RFC5969_OPTIONS)
+        { "option-6rd", "BB6Ia",		&dhcp_universe, 212, 1 },
+#endif
+#if defined(RFC5986_OPTIONS)
+	{"v4-access-domain", "d",		&dhcp_universe, 213, 1 },
 #endif
 	{ NULL, NULL, NULL, 0, 0 }
 };
@@ -397,8 +435,7 @@ static struct option dhcpv6_options[] = {
 
 	/* Note that 35 is not assigned. */
 
-	/* Not yet considering for inclusion. */
-#if 0
+#if defined(RFC4776_OPTIONS)
 			/* RFC4776 OPTIONS */
 
 	{ "geoconf-civic", "X",			&dhcpv6_universe, 36, 1 },
@@ -432,18 +469,20 @@ static struct option dhcpv6_options[] = {
 	{ "fqdn", "Efqdn6-if-you-see-me-its-a-bug-bug-bug.",
 						&dhcpv6_universe, 39, 1 },
 
-	/* Not yet considering for inclusion. */
-#if 0
-			/* draft-ietf-dhc-paa-option-05 */
+
+			/* RFC5192 */
+#if defined(RFC5192_OPTIONS)
 	{ "pana-agent", "6A",			&dhcpv6_universe, 40, 1 },
+#endif
 
 			/* RFC4833 OPTIONS */
-
+#if defined(RFC4833_OPTIONS)
 	{ "new-posix-timezone", "t",		&dhcpv6_universe, 41, 1 },
 	{ "new-tzdb-timezone", "t",		&dhcpv6_universe, 42, 1 },
+#endif
 
 			/* RFC4994 OPTIONS */
-
+#if defined(RFC4994_OPTIONS)
 	{ "ero", "SA",				&dhcpv6_universe, 43, 1 },
 #endif
 
@@ -455,13 +494,90 @@ static struct option dhcpv6_options[] = {
 	{ "lq-relay-data", "6X",		&dhcpv6_universe, 47, 1 },
 	{ "lq-client-link", "6A",		&dhcpv6_universe, 48, 1 },
 
+			/* RFC5223 OPTIONS */
+#if defined(RFC5223_OPTIONS)
+	{ "v6-lost", "d",			&dhcpv6_universe, 51, 1 },
+#endif
+
+			/* RFC5417 OPTIONS */
+#if defined(RFC5417_OPTIONS)
+	{ "capwap-ac-v6", "6a",			&dhcpv6_universe, 52, 1 },
+#endif
+
+			/* RFC5460 OPTIONS */
+#if defined(RFC5460_OPTIONS)
+	{ "relay-id", "X",			&dhcpv6_universe, 53, 1 },
+#endif
+
+			/* RFC5986 OPTIONS */
+#if defined(RFC5986_OPTIONS)
+	{ "v6-access-domain", "d",		&dhcpv6_universe, 57, 1 },
+#endif
+
+			/* RFC6011 OPTIONS */
+#if defined(RFC6011_OPTIONS)
+	{ "sip-ua-cs-list", "D",		&dhcpv6_universe, 58, 1 },
+#endif
+
+			/* RFC5970 OPTIONS */
+#if defined(RFC5970_OPTIONS)
+	{ "bootfile-url", "t",			&dhcpv6_universe, 59, 1 },
+	{ "bootfile-param", "X",		&dhcpv6_universe, 60, 1 },
+	{ "client-arch-type", "SA",		&dhcpv6_universe, 61, 1 },
+	{ "nii", "BBB",				&dhcpv6_universe, 62, 1 },
+#endif
+
+			/* RFC6334 OPTIONS */
+#if defined(RFC6334_OPTIONS)
+	{ "aftr-name", "d",			&dhcpv6_universe, 64, 1 },
+#endif
+
+			/* RFC6440 OPTIONS */
+#if defined(RFC6440_OPTIONS)
+	{ "erp-local-domain-name", "d",		&dhcpv6_universe, 65, 1 },
+#endif
+
+			/* RFC6731 OPTIONS */
+#if defined(RFC6731_OPTIONS)
+	{ "rdnss-selection", "6BD",		&dhcpv6_universe, 74, 1 },
+#endif
+
+			/* RFC6939 OPTIONS */
+#if defined(RFC6939_OPTIONS)
+	{ "client-linklayer-addr", "X",		&dhcpv6_universe, 79, 1 },
+#endif
+
+			/* RFC6977 OPTIONS */
+#if defined(RFC6977_OPTIONS)
+	{ "link-address", "6",			&dhcpv6_universe, 80, 1 },
+#endif
+
+			/* RFC7083 OPTIONS */
+#if defined(RFC7083_OPTIONS)
+	{ "solmax-rt", "L",			&dhcpv6_universe, 82, 1 },
+	{ "inf-max-rt", "L",			&dhcpv6_universe, 83, 1 },
+#endif
+#if defined(RFC7710_OPTIONS)
+	{ "v6-captive-portal", "t",		&dhcpv6_universe, 103, 1 },
+#endif
+#if defined(RFC6153_OPTIONS)
+	{ "ipv6-address-andsf", "6A",		&dhcpv6_universe, 143, 1 },
+#endif
+
+			/* RFC7341 OPTIONS */
+#if defined(RFC7341_OPTIONS)
+	{ "dhcpv4-msg", "X",			&dhcpv6_universe, 87, 1 },
+	{ "dhcp4-o-dhcp6-server", "6A",		&dhcpv6_universe, 88, 1 },
+#endif
+
 	{ NULL, NULL, NULL, 0, 0 }
 };
 
 struct enumeration_value dhcpv6_duid_type_values[] = {
-	{ "duid-llt",	DUID_LLT }, /* Link-Local Plus Time */
-	{ "duid-en",	DUID_EN },  /* DUID based upon enterprise-ID. */
-	{ "duid-ll",	DUID_LL },  /* DUID from Link Local address only. */
+	{ "duid-llt",	DUID_LLT },  /* Link-Local Plus Time */
+	{ "duid-en",	DUID_EN },   /* DUID based upon enterprise-ID. */
+	{ "duid-ll",	DUID_LL },   /* DUID from Link Local address only. */
+	{ "duid-uuid",	DUID_UUID }, /* DUID based upon UUID */
 	{ NULL, 0 }
 };
 
@@ -483,6 +599,7 @@ struct enumeration_value dhcpv6_status_code_values[] = {
 	{ "MalformedQuery", 8 }, /* Leasequery not valid.		*/
 	{ "NotConfigured", 9 }, /* The target address is not in config.	*/
 	{ "NotAllowed",  10 }, /* Server doesn't allow the leasequery.	*/
+	{ "QueryTerminated", 11 }, /* Leasequery terminated.		*/
 	{ NULL, 0 }
 };
 
@@ -495,6 +612,9 @@ struct enumeration dhcpv6_status_codes = {
 struct enumeration_value lq6_query_type_values[] = {
 	{ "query-by-address", 1 },
 	{ "query-by-clientid", 2 },
+	{ "query-by-relay-id", 3 },
+	{ "query-by-link-address", 4 },
+	{ "query-by-remote-id", 5 },
 	{ NULL, 0 }
 };
 
@@ -520,6 +640,12 @@ struct enumeration_value dhcpv6_message_values[] = {
 	{ "RELAY-REPL", 13 },
 	{ "LEASEQUERY", 14 },
 	{ "LEASEQUERY-REPLY", 15 },
+	{ "LEASEQUERY-DONE", 16 },
+	{ "LEASEQUERY-DATA", 17 },
+	{ "RECONFIGURE-REQUEST", 18 },
+	{ "RECONFIGURE-REPLY", 19 },
+	{ "DHCPV4-QUERY", 20 },
+	{ "DHCPV4-RESPONSE", 21 },
 	{ NULL, 0 }
 };
 
@@ -540,7 +666,13 @@ const char *dhcpv6_type_names[] = {
 	"Relay-forward",
 	"Relay-reply",
 	"Leasequery",
-	"Leasequery-reply"
+	"Leasequery-reply",
+	"Leasequery-done",
+	"Leasequery-data",
+	"Reconfigure-request",
+	"Reconfigure-reply",
+	"Dhcpv4-query",
+	"Dhcpv4-response"
 };
 const int dhcpv6_type_name_max =
 	(sizeof(dhcpv6_type_names) / sizeof(dhcpv6_type_names[0]));
@@ -560,7 +692,9 @@ static struct option vsio_options[] = {
 struct universe isc6_universe;
 static struct option isc6_options[] = {
 	{ "media", "t",				&isc6_universe,     1, 1 },
-	{ "update-assist", "X",			&isc6_universe,	    2, 1 },
+	{ "update-assist", "X",			&isc6_universe,     2, 1 },
+	{ "4o6-interface", "t",			&isc6_universe, 60000, 1 },
+	{ "4o6-source-address", "6",		&isc6_universe, 60001, 1 },
 	{ NULL, NULL, NULL, 0, 0 }
 };
 
