@@ -3,7 +3,9 @@
    Routines for manipulating hash tables... */
 
 /*
- * Copyright (c) 2004-2008 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2014-2015 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2009-2010 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -24,23 +26,15 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
-#ifndef lint
-static char copyright[] =
-"$Id: hash.c,v 1.11.16.6 2009/07/23 21:43:35 sar Exp $ Copyright (c) 2004-2008 Internet Systems Consortium.  All rights reserved.\n";
-#endif /* not lint */
+#include "dhcpd.h"
 
 #include <omapip/omapip_p.h>
+#include <limits.h>
 #include <ctype.h>
 
-static INLINE unsigned
+static unsigned
 find_length(const void *key,
 	    unsigned (*do_hash)(const void *, unsigned, unsigned))
 {
@@ -51,8 +45,13 @@ find_length(const void *key,
 	if (do_hash == do_ip4_hash)
 		return 4;
 
-	log_fatal("Impossible condition at %s:%d.", MDL);
-	return 0; /* Silence compiler warnings. */
+	log_debug("Unexpected hash function at %s:%d.", MDL);
+	/*
+	 * If we get a hash function we don't specifically expect
+	 * return a length of 0, this covers the case where a client
+	 * id has a length of 0.
+	 */
+	return 0;
 }
 
 int new_hash_table (tp, count, file, line)
@@ -103,13 +102,14 @@ void free_hash_table (tp, file, line)
 	const char *file;
 	int line;
 {
-	int i;
-	struct hash_bucket *hbc, *hbn = (struct hash_bucket *)0;
 	struct hash_table *ptr = *tp;
 
 #if defined (DEBUG_MEMORY_LEAKAGE) || \
 		defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
-	for (i = 0; i < ptr -> hash_count; i++) {
+	int i;
+	struct hash_bucket *hbc, *hbn = (struct hash_bucket *)0;
+
+	for (i = 0; ptr != NULL && i < ptr -> hash_count; i++) {
 	    for (hbc = ptr -> buckets [i]; hbc; hbc = hbn) {
 		hbn = hbc -> next;
 		if (ptr -> dereferencer && hbc -> value)
@@ -123,7 +123,7 @@ void free_hash_table (tp, file, line)
 	}
 #endif
 
-	dfree ((VOIDPTR)ptr, MDL);
+	dfree((void *)ptr, MDL);
 	*tp = (struct hash_table *)0;
 }
 
@@ -197,8 +197,9 @@ void free_hash_bucket (ptr, file, line)
 	const char *file;
 	int line;
 {
-	struct hash_bucket *hp;
 #if defined (DEBUG_MALLOC_POOL)
+	struct hash_bucket *hp;
+
 	for (hp = free_hash_buckets; hp; hp = hp -> next) {
 		if (hp == ptr) {
 			log_error ("hash bucket freed twice!");
@@ -297,21 +298,25 @@ do_id_hash(const void *name, unsigned len, unsigned size)
 	if (len == 0)
 		return 0;
 
-	/* The switch indexes our starting position into the do/while loop,
-	 * taking up the remainder after hashing in all the other bytes in
-	 * threes.
+	/*
+	 * The switch handles our starting conditions, then we hash the
+	 * remaining bytes in groups of 3
 	 */
+	   
 	switch (len % 3) {
-		do {
-	      case 0:
-			accum ^= *s++ << 16;
-	      case 2:
-			accum ^= *s++ << 8;
-	      case 1:
-			accum ^= *s++;
-		} while (s < end);
-
+	case 0:
 		break;
+	case 2:
+		accum ^= *s++ << 8;
+	case 1:
+		accum ^= *s++;
+		break;
+	}
+
+	while (s < end) {
+		accum ^= *s++ << 16;
+		accum ^= *s++ << 8;
+		accum ^= *s++;
 	}
 
 	return accum % size;
@@ -352,7 +357,7 @@ hash_report(struct hash_table *table)
 		return (unsigned char *) "No table.";
 
 	if (table->hash_count == 0)
-		return (char *) "Invalid hash table.";
+		return (unsigned char *) "Invalid hash table.";
 
 	for (i = 0 ; i < table->hash_count ; i++) {
 		curlen = 0;
@@ -381,9 +386,10 @@ hash_report(struct hash_table *table)
 	    pct > 2147483647 ||
 	    minlen > 2147483647 ||
 	    maxlen > 2147483647)
-		return (char *) "Report out of range for display.";
+		return (unsigned char *) "Report out of range for display.";
 
-	sprintf(retbuf, "Contents/Size (%%): %u/%u (%u%%). Min/max: %u/%u",
+	sprintf((char *)retbuf, 
+		"Contents/Size (%%): %u/%u (%u%%). Min/max: %u/%u",
 		contents, table->hash_count, pct, minlen, maxlen);
 
 	return retbuf;
