@@ -3,7 +3,7 @@
    DHCP/BOOTP Relay Agent. */
 
 /*
- * Copyright(c) 2004-2014 by Internet Systems Consortium, Inc.("ISC")
+ * Copyright(c) 2004-2016 by Internet Systems Consortium, Inc.("ISC")
  * Copyright(c) 1997-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -30,6 +30,7 @@
 #include <syslog.h>
 #include <signal.h>
 #include <sys/time.h>
+#include <isc/file.h>
 
 TIME default_lease_time = 43200; /* 12 hours... */
 TIME max_lease_time = 86400; /* 24 hours... */
@@ -60,6 +61,7 @@ int server_packets_relayed = 0;	/* Packets relayed from server to client. */
 int client_packet_errors = 0;	/* Errors sending packets to clients. */
 
 int add_agent_options = 0;	/* If nonzero, add relay agent options. */
+int add_rfc3527_suboption = 0;	/* If nonzero, add RFC3527 link selection sub-option. */
 
 int agent_option_errors = 0;    /* Number of packets forwarded without
 				   agent options because there was no room. */
@@ -99,6 +101,8 @@ struct server_list {
 	struct sockaddr_in to;
 } *servers;
 
+struct interface_info *uplink = NULL;
+
 #ifdef DHCPv6
 struct stream_list {
 	struct stream_list *next;
@@ -133,22 +137,25 @@ static int strip_relay_agent_options(struct interface_info *,
 				     struct dhcp_packet *, unsigned);
 
 static const char copyright[] =
-"Copyright 2004-2014 Internet Systems Consortium.";
+"Copyright 2004-2016 Internet Systems Consortium.";
 static const char arr[] = "All rights reserved.";
 static const char message[] =
 "Internet Systems Consortium DHCP Relay Agent";
 static const char url[] =
 "For info, please visit https://www.isc.org/software/dhcp/";
 
+char *progname;
+
 #ifdef DHCPv6
 #define DHCRELAY_USAGE \
-"Usage: dhcrelay [-4] [-d] [-q] [-a] [-D]\n"\
+"Usage: %s [-4] [-d] [-q] [-a] [-D]\n"\
 "                     [-A <length>] [-c <hops>] [-p <port>]\n" \
 "                     [-pf <pid-file>] [--no-pid]\n"\
 "                     [-m append|replace|forward|discard]\n" \
 "                     [-i interface0 [ ... -i interfaceN]\n" \
+"                     [-U interface]\n" \
 "                     server0 [ ... serverN]\n\n" \
-"       dhcrelay -6   [-d] [-q] [-I] [-c <hops>] [-p <port>]\n" \
+"       %s -6   [-d] [-q] [-I] [-c <hops>] [-p <port>]\n" \
 "                     [-pf <pid-file>] [--no-pid]\n" \
 "                     [-s <subscriber-id>]\n" \
 "                     -l lower0 [ ... -l lowerN]\n" \
@@ -157,15 +164,50 @@ static const char url[] =
 "       upper (server link): [address%%]interface"
 #else
 #define DHCRELAY_USAGE \
-"Usage: dhcrelay [-d] [-q] [-a] [-D] [-A <length>] [-c <hops>] [-p <port>]\n" \
+"Usage: %s [-d] [-q] [-a] [-D] [-A <length>] [-c <hops>] [-p <port>]\n" \
 "                [-pf <pid-file>] [--no-pid]\n" \
 "                [-m append|replace|forward|discard]\n" \
 "                [-i interface0 [ ... -i interfaceN]\n" \
+"                [-U interface]\n" \
 "                server0 [ ... serverN]\n\n"
 #endif
 
-static void usage() {
-	log_fatal(DHCRELAY_USAGE);
+/*!
+ *
+ * \brief Print the generic usage message
+ *
+ * If the user has provided an incorrect command line print out
+ * the description of the command line.  The arguments provide
+ * a way for the caller to request more specific information about
+ * the error be printed as well.  Mostly this will be that some
+ * comamnd doesn't include its argument.
+ *
+ * \param sfmt - The basic string and format for the specific error
+ * \param sarg - Generally the offending argument from the comamnd line.
+ *
+ * \return Nothing
+ */
+static const char use_noarg[] = "No argument for command: %s";
+#ifdef DHCPv6
+static const char use_badproto[] = "Protocol already set, %s inappropriate";
+static const char use_v4command[] = "Command not used for DHCPv6: %s";
+static const char use_v6command[] = "Command not used for DHCPv4: %s";
+#endif
+
+static void
+usage(const char *sfmt, const char *sarg) {
+
+	/* If desired print out the specific error message */
+#ifdef PRINT_SPECIFIC_CL_ERRORS
+	if (sfmt != NULL)
+		log_error(sfmt, sarg);
+#endif
+
+	log_fatal(DHCRELAY_USAGE,
+#ifdef DHCPv6
+		  isc_file_basename(progname),
+#endif
+		  isc_file_basename(progname));
 }
 
 int 
@@ -184,6 +226,12 @@ main(int argc, char **argv) {
 	int local_family_set = 0;
 #endif
 
+#ifdef OLD_LOG_NAME
+	progname = "dhcrelay";
+#else
+	progname = argv[0];
+#endif
+
 	/* Make sure that file descriptors 0(stdin), 1,(stdout), and
 	   2(stderr) are open. To do this, we assume that when we
 	   open a file the lowest available file descriptor is used. */
@@ -197,7 +245,7 @@ main(int argc, char **argv) {
 	else if (fd != -1)
 		close(fd);
 
-	openlog("dhcrelay", LOG_NDELAY, LOG_DAEMON);
+	openlog(isc_file_basename(progname), DHCP_LOG_OPTIONS, LOG_DAEMON);
 
 #if !defined(DEBUG)
 	setlogmask(LOG_UPTO(LOG_INFO));
@@ -223,13 +271,13 @@ main(int argc, char **argv) {
 		if (!strcmp(argv[i], "-4")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_badproto, "-4");
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
 		} else if (!strcmp(argv[i], "-6")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_badproto, "-6");
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
@@ -241,29 +289,29 @@ main(int argc, char **argv) {
 			quiet_interface_discovery = 1;
 		} else if (!strcmp(argv[i], "-p")) {
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			local_port = validate_port(argv[i]);
 			log_debug("binding to user-specified port %d",
 				  ntohs(local_port));
 		} else if (!strcmp(argv[i], "-c")) {
 			int hcount;
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			hcount = atoi(argv[i]);
 			if (hcount <= 255)
 				max_hop_count= hcount;
 			else
-				usage();
+				usage("Bad hop count to -c: %s", argv[i]);
  		} else if (!strcmp(argv[i], "-i")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
 #endif
 			if (++i == argc) {
-				usage();
+				usage(use_noarg, argv[i-1]);
 			}
 			if (strlen(argv[i]) >= sizeof(tmp->name)) {
 				log_fatal("%s: interface name too long "
@@ -282,7 +330,7 @@ main(int argc, char **argv) {
 		} else if (!strcmp(argv[i], "-a")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
@@ -291,13 +339,13 @@ main(int argc, char **argv) {
 		} else if (!strcmp(argv[i], "-A")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
 #endif
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 
 			dhcp_max_agent_option_packet_length = atoi(argv[i]);
 
@@ -308,13 +356,13 @@ main(int argc, char **argv) {
 		} else if (!strcmp(argv[i], "-m")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
 #endif
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			if (!strcasecmp(argv[i], "append")) {
 				agent_relay_mode = forward_and_append;
 			} else if (!strcasecmp(argv[i], "replace")) {
@@ -324,11 +372,41 @@ main(int argc, char **argv) {
 			} else if (!strcasecmp(argv[i], "discard")) {
 				agent_relay_mode = discard;
 			} else
-				usage();
+				usage("Unknown argument to -m: %s", argv[i]);
+		} else if (!strcmp(argv [i], "-U")) {
+			if (++i == argc)
+				usage(use_noarg, argv[i-1]);
+
+			if (uplink) {
+				usage("more than one uplink (-u) specified: %s"
+				      ,argv[i]);
+			}
+
+			/* Allocate the uplink interface */
+			status = interface_allocate(&uplink, MDL);
+			if (status != ISC_R_SUCCESS) {
+				log_fatal("%s: uplink interface_allocate: %s",
+					 argv[i], isc_result_totext(status));
+			}
+		
+			if (strlen(argv[i]) >= sizeof(uplink->name)) {
+				log_fatal("%s: uplink name too long,"
+					  " it cannot exceed: %ld characters",
+					  argv[i], (long)(sizeof(uplink->name) - 1));
+			}
+
+			uplink->name[sizeof(uplink->name) - 1] = 0x00;
+			strncpy(uplink->name, argv[i],
+				sizeof(uplink->name) - 1);
+			interface_snorf(uplink, INTERFACE_REQUESTED);
+
+			/* Turn on -a, in case they don't do so explicitly */
+			add_agent_options = 1;
+			add_rfc3527_suboption = 1;
 		} else if (!strcmp(argv[i], "-D")) {
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
@@ -337,48 +415,48 @@ main(int argc, char **argv) {
 #ifdef DHCPv6
 		} else if (!strcmp(argv[i], "-I")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_v6command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
 			use_if_id = ISC_TRUE;
 		} else if (!strcmp(argv[i], "-l")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_v6command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
 			if (downstreams != NULL)
 				use_if_id = ISC_TRUE;
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			sl = parse_downstream(argv[i]);
 			sl->next = downstreams;
 			downstreams = sl;
 		} else if (!strcmp(argv[i], "-u")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_v6command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			sl = parse_upstream(argv[i]);
 			sl->next = upstreams;
 			upstreams = sl;
 		} else if (!strcmp(argv[i], "-s")) {
 			if (local_family_set && (local_family == AF_INET)) {
-				usage();
+				usage(use_v6command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET6;
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			dhcrelay_sub_id = argv[i];
 #endif
 		} else if (!strcmp(argv[i], "-pf")) {
 			if (++i == argc)
-				usage();
+				usage(use_noarg, argv[i-1]);
 			path_dhcrelay_pid = argv[i];
 			no_dhcrelay_pid = ISC_TRUE;
 		} else if (!strcmp(argv[i], "--no-pid")) {
@@ -388,17 +466,21 @@ main(int argc, char **argv) {
 			exit(0);
 		} else if (!strcmp(argv[i], "--help") ||
 			   !strcmp(argv[i], "-h")) {
-			log_info(DHCRELAY_USAGE);
+			log_info(DHCRELAY_USAGE,
+#ifdef DHCPv6
+				 isc_file_basename(progname),
+#endif
+				 isc_file_basename(progname));
 			exit(0);
  		} else if (argv[i][0] == '-') {
-			usage();
+			usage("Unknown command: %s", argv[i]);
  		} else {
 			struct hostent *he;
 			struct in_addr ia, *iap = NULL;
 
 #ifdef DHCPv6
 			if (local_family_set && (local_family == AF_INET6)) {
-				usage();
+				usage(use_v4command, argv[i]);
 			}
 			local_family_set = 1;
 			local_family = AF_INET;
@@ -510,7 +592,7 @@ main(int argc, char **argv) {
 		if (upstreams == NULL || downstreams == NULL) {
 			log_info("Must specify at least one lower "
 				 "and one upper interface.\n");
-			usage();
+			usage(NULL, NULL);
 		}
 
 		/* Set up the initial dhcp option universe. */
@@ -591,9 +673,12 @@ main(int argc, char **argv) {
 		dhcpv6_packet_handler = do_packet6;
 #endif
 
+#if defined(ENABLE_GENTLE_SHUTDOWN)
+	/* no signal handlers until we deal with the side effects */
         /* install signal handlers */
 	signal(SIGINT, dhcp_signal_handler);   /* control-c */
 	signal(SIGTERM, dhcp_signal_handler);  /* kill */
+#endif
 
 	/* Start dispatching packets and timeouts... */
 	dispatch();
@@ -703,7 +788,8 @@ do_relay4(struct interface_info *ip, struct dhcp_packet *packet,
 		return;
 
 	/* Add relay agent options if indicated.   If something goes wrong,
-	   drop the packet. */
+	 * drop the packet.  Note this may set packet->giaddr if RFC3527
+	 * is enabled. */
 	if (!(length = add_relay_agent_options(ip, packet, length,
 					       ip->addresses[0])))
 		return;
@@ -945,6 +1031,7 @@ add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
 	int is_dhcp = 0, mms;
 	unsigned optlen;
 	u_int8_t *op, *nextop, *sp, *max, *end_pad = NULL;
+	int adding_link_select;
 
 	/* If we're not adding agent options to packets, we can skip
 	   this. */
@@ -958,6 +1045,10 @@ add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
 
 	max = ((u_int8_t *)packet) + dhcp_max_agent_option_packet_length;
 
+	/* Add link selection suboption if enabled and we're the first relay */
+	adding_link_select = (add_rfc3527_suboption
+			      && (packet->giaddr.s_addr == 0));
+
 	/* Commence processing after the cookie. */
 	sp = op = &packet->options[4];
 
@@ -1087,6 +1178,10 @@ add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
 		optlen += ip->remote_id_len + 2;    /* RAI_REMOTE_ID + len */
 	}
 
+	if (adding_link_select) {
+		optlen += 6;
+	}
+
 	/* We do not support relay option fragmenting(multiple options to
 	 * support an option data exceeding 255 bytes).
 	 */
@@ -1118,6 +1213,19 @@ add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
 			memcpy(sp, ip->remote_id, ip->remote_id_len);
 			sp += ip->remote_id_len;
 		}
+
+		/* RFC3527: Use the inbound packet's interface address in
+		 * the link selection suboption and set the outbound giaddr
+		 * to the uplink address. */
+		if (adding_link_select) {
+			*sp++ = RAI_LINK_SELECT;
+			*sp++ = 4u;
+			memcpy(sp, &giaddr.s_addr, 4);
+			sp += 4;
+			packet->giaddr = uplink->addresses[0];
+			log_debug ("Adding link selection suboption"
+				   " with addr: %s", inet_ntoa(giaddr));
+		}
 	} else {
 		++agent_option_errors;
 		log_error("No room in packet (used %d of %d) "
@@ -1175,8 +1283,7 @@ parse_downstream(char *arg) {
 		*iid++ = '\0';
 	}
 	if (strlen(ifname) >= sizeof(ifp->name)) {
-		log_error("Interface name '%s' too long", ifname);
-		usage();
+		usage("Interface name '%s' too long", ifname);
 	}
 
 	/* Don't declare twice. */
@@ -1189,8 +1296,8 @@ parse_downstream(char *arg) {
 	/* Share with up side? */
 	for (up = upstreams; up; up = up->next) {
 		if (strcmp(ifname, up->ifp->name) == 0) {
-			log_info("Interface '%s' is both down and up.",
-				 ifname);
+			log_info("parse_downstream: Interface '%s' is "
+				 "both down and up.", ifname);
 			ifp = up->ifp;
 			break;
 		}
@@ -1208,8 +1315,8 @@ parse_downstream(char *arg) {
 			interface_dereference(&interfaces, MDL);
 		}
 		interface_reference(&interfaces, ifp, MDL);
-		ifp->flags |= INTERFACE_REQUESTED | INTERFACE_DOWNSTREAM;
 	}
+	ifp->flags |= INTERFACE_REQUESTED | INTERFACE_DOWNSTREAM;
 
 	/* New downstream. */
 	dp = (struct stream_list *) dmalloc(sizeof(*dp), MDL);
@@ -1260,6 +1367,8 @@ parse_upstream(char *arg) {
 	}
 	for (dp = downstreams; dp; dp = dp->next) {
 		if (strcmp(ifname, dp->ifp->name) == 0) {
+			log_info("parse_upstream: Interface '%s' is "
+				 "both down and up.", ifname);
 			ifp = dp->ifp;
 			break;
 		}
@@ -1277,8 +1386,8 @@ parse_upstream(char *arg) {
 			interface_dereference(&interfaces, MDL);
 		}
 		interface_reference(&interfaces, ifp, MDL);
-		ifp->flags |= INTERFACE_REQUESTED | INTERFACE_UPSTREAM;
 	}
+	ifp->flags |= INTERFACE_REQUESTED | INTERFACE_UPSTREAM;
 
 	/* New upstream. */
 	up = (struct stream_list *) dmalloc(sizeof(*up), MDL);
@@ -1346,6 +1455,13 @@ setup_streams(void) {
 		if (up->ifp->v6address_count == 0)
 			log_fatal("Interface '%s' has no IPv6 addresses.",
 				  up->ifp->name);
+
+		/* RFC 3315 Sec 20 - "If the relay agent relays messages to
+		 * the All_DHCP_Servers address or other multicast addresses,
+		 * it sets the Hop Limit field to 32." */
+		if (IN6_IS_ADDR_MULTICAST(&up->link.sin6_addr)) {
+			set_multicast_hop_limit(up->ifp, HOP_COUNT_LIMIT);
+		}
 	}
 }
 
@@ -1382,6 +1498,7 @@ process_up6(struct packet *packet, struct stream_list *dp) {
 	      case DHCPV6_INFORMATION_REQUEST:
 	      case DHCPV6_RELAY_FORW:
 	      case DHCPV6_LEASEQUERY:
+	      case DHCPV6_DHCPV4_QUERY:
 		log_info("Relaying %s from %s port %d going up.",
 			 dhcpv6_type_names[packet->dhcpv6_msg_type],
 			 piaddr(packet->client_addr),
@@ -1393,6 +1510,7 @@ process_up6(struct packet *packet, struct stream_list *dp) {
 	      case DHCPV6_RECONFIGURE:
 	      case DHCPV6_RELAY_REPL:
 	      case DHCPV6_LEASEQUERY_REPLY:
+	      case DHCPV6_DHCPV4_RESPONSE:
 		log_info("Discarding %s from %s port %d going up.",
 			 dhcpv6_type_names[packet->dhcpv6_msg_type],
 			 piaddr(packet->client_addr),
@@ -1611,6 +1729,7 @@ process_down6(struct packet *packet) {
 	      case DHCPV6_RECONFIGURE:
 	      case DHCPV6_RELAY_FORW:
 	      case DHCPV6_LEASEQUERY_REPLY:
+	      case DHCPV6_DHCPV4_RESPONSE:
 		log_info("Relaying %s to %s port %d down.",
 			 dhcpv6_type_names[msg->msg_type],
 			 piaddr(peer),
@@ -1626,6 +1745,7 @@ process_down6(struct packet *packet) {
 	      case DHCPV6_DECLINE:
 	      case DHCPV6_INFORMATION_REQUEST:
 	      case DHCPV6_LEASEQUERY:
+	      case DHCPV6_DHCPV4_QUERY:
 		log_info("Discarding %s to %s port %d down.",
 			 dhcpv6_type_names[msg->msg_type],
 			 piaddr(peer),
@@ -1717,5 +1837,9 @@ dhcp_set_control_state(control_object_state_t oldstate,
 		       control_object_state_t newstate) {
 	if (newstate != server_shutdown)
 		return ISC_R_SUCCESS;
+
+	if (no_pid_file == ISC_FALSE)
+		(void) unlink(path_dhcrelay_pid);
+
 	exit(0);
 }
